#!/usr/bin/perl 
# Check that the above line points to where perl really is!
#
#    @(#) ingwho.perl 1.23@(#) 09 Jul 1995
#
#    INGWHO - a script to list and/or remove all Ingres user sessions in
#             the current installation, with or without prompting for
#             confirmation. Designed to support a multi-server installation.
#
#    Usage: 
#     ingwho [[-format] [-remove] [-noprompt]] show current Ingres
#            user  sessions
#
#    Author: Mark Merritt
#    Date:   Tue Sep 28 11:20:24 EDT 1993
#
#
# put a 'whoami' and a 'ps' command that works on your system here
$WHOAMI='/usr/ucb/whoami';
$PS='ps -ef';

# right number of arguments?
if( $#ARGV > 2 )
{
   &usage;
}

# initialization
$remove = 0;
$noprompt = 0;
$format = 0;
$total_sessions =  0;
$args = 0;

# Determine what parameters we have - I tried using getopt here but found it 
# rather clumsy
#
foreach $arg (@ARGV)
{
   if( $arg eq '-format' )
   {  
       $args++;
       $format = 1;
   }
   elsif( $arg  eq '-remove' )
   {
       $args++;
       $remove = 1;
   }
   elsif( $arg  eq '-noprompt' )
   {
       $args++;
       $noprompt = 1;
   }
}

# accounted for all arguments?
if( --$args != $#ARGV )
{
   &usage;
}

#  String to identify our monitor sessions with so they can be ignored
$myname = `$WHOAMI`;
chop($myname);
$monitorstring = sprintf("%-24s\\) cs_state: CS_COMPUTABLE cs_mask:", $myname );

## first get a list of all registered DBMS servers
##
@SERVERS=`runiinamu iidbms | sed \'s\/ *(.*)\/\/\' | sort | uniq`;

$dbmsr_count=$#SERVERS + 1;
#
##
## go through the list of registered DBMS servers

##
print "Public DBMS Servers:\n";

&printServers(@SERVERS);

#
# All the dbms servers known to this installation. See if any of these are
# are NOT in  our original list of public servers.

$csPidCount = 0;
@csreport = `csreport | grep \"inuse 1\"`;
if (!defined  @csreport) 
{
   &printTotal();
   print 
      "No return from csreport. Skipping check for private and star servers\n";
   exit;
}

CSSERVER:
foreach $server (@csreport)
{
   @csline = split(' ',$server);
   # potential dbms server
   if( $csline[6] ne "," )
   {

      chop($csline[6]);
      # see if this is in our list of public servers
      foreach $server (@SERVERS)
      {
         if( "$csline[6]" eq "$server" )
         {
	     next CSSERVER;
         }
      }
      $csServerList{$csline[3] - ","} .= $csline[6];
      $csPidCount++;
   }
}

# make sure this is REALLY a dbms server
if( $csPidCount > 0 )
{
   # get information for any private servers. First get the pids of all dbms
   # servers running on the system, then see(below) if any of these are part
   # of THIS installation, but are not in our original list of public servers.
   
   $dbmsPidCount = 0;

   # had some trouble using more than two pipes! had to adapt to that
   @dbms = `$PS | grep dbms | grep -v grep`;
   foreach $server (@dbms)
   {
      @dbmsArray = split(' ',$server);
      $pId =  $csServerList{$dbmsArray[1]};
      if( defined( $pId ))
      {
         $PSERVERS[$dbmsPidCount++] = sprintf("%s\n",$pId);
      }  
   }  
}

# OK - if there are any private servers, now we have a list of them
if( $#PSERVERS >= 0 )
{
   print "\nPrivate DBMS Servers:\n";
   &printServers(@PSERVERS);
}

# now see if there are any star servers running in this installation
@stars = `$PS | grep iistar | grep -v grep`;
$dbmsPidCount = 0;
foreach $server (@stars)
{
   @dbmsArray = split(' ',$server);
   $pId =  $csServerList{$dbmsArray[1]};
   if( defined( $pId ))
   {
	$SSERVERS[$dbmsPidCount++] = sprintf("%s\n",$pId);
   }
}

if( $#SSERVERS >= 0 )
{
   print "\nStar Servers:\n";
   &printServers(@SSERVERS);
}

&printTotal();

# finished
exit 0;

sub
printTotal
{
# strip the path from the executable name 
@exe = split(/\//,$0);

printf( "\n%s: %d user %s\n", $exe[$#exe], $total_sessions,
            $total_sessions == 1 ? "session" : "sessions" );
}

sub
usage
{
   print "Usage:\n";
   print 
    "$0 [[-format] [-remove] [-noprompt]] show current Ingres user sessions\n";
   print "   Default display format is one-line from iimonitor\n";
   print "   -format   - Show format of sessions in iimonitor\n";
   print "   -remove   - Remove user sessions, prompting for confirmation\n";
   print "               Sessions are removed on a per-server basis\n";
   print "   -noprompt - Remove all user sessions without prompting\n";
   exit 1;
}

sub
printServers
{

   foreach $dbms (@_)
   {
      chop($dbms);
      printf("\*\*\* Server: %s \*\*\*\n",$dbms);
   
      $sessionList=`iimonitor $dbms << !
show sessions
quit
!`;
   
      $sessionList =~ y/\000-\037//d; # get rid of carriage returns and stuff
      $sessionList =~ s/IIMONITOR> //g;
      
      @SESSIONS = split( /[Ss]ession /, $sessionList );
      
      #
      # this whole script is only as good as the strings we're using here to 
      # strip the sessions we don't care about from the output of iimonitor
      
      @SESSIONS = grep( !(/Write Behind / | /Fast Commit/ | /Event Thread/ 
			  | /idle job/), @SESSIONS );
      shift @SESSIONS;
      
      #
      #  Previous versions relied on the assumption here is that the monitor 
      #  session is the only one that is CS_COMPUTABLE. Unfortunately, this 
      #  isn't always true! We make the assumption that the last session is 
      #  our monitor session. The hope is there won't be time for another user
      #  to connect in the split nano second between the time we establish our
      #  connection and the time we create our session list.
      $#SESSIONS -= 1;
       
      $session_count = ( $#SESSIONS + 1 );
      $total_sessions +=  $session_count;
      if ( $session_count > 0 )
      {
   	 foreach $session (@SESSIONS)
   	 {
	    $session =~ y/ / /s;
	    print $session,"\n";;
      
	    if( $format )
	    {
	       @sessNo = split(' ', $session );
	       $form =`iimonitor $dbms << !
format $sessNo[0]
quit
!`;
	       $form =~  s/IIMONITOR> //g ;
	       print $form;
	    }
   	 }
       
         printf( "%d %s\n",$session_count, 
            $session_count == 1 ? "session" : "sessions"
            );
      
   	 # remove the sessions if we have been requested to do so. There is a 
   	 # timing issue here that can cause some problems. If another session
   	 # connects AFTER we create our session list, it won't get removed!
   	 if($remove)
   	 {
	    if(!$noprompt)
	    {
		  print 
	     "Caution: Removing sessions with releases prior to 6.4/05 may cause\n";
		  print 
   	"severe problems in your installation! Really remove these sessions?(y/n) ";
		  $prompt1 = <STDIN>;
	    }
	       else
	    {
		  $prompt1="ye";
	    }
   	 
	    if($prompt1)
	    {
	       chop($prompt1);
	       $prompt1 =~ y/A-Z/a-z/;  
	       if(($prompt1 eq "y") || ($prompt1 eq "yes"))
	       {
		  foreach $session (@SESSIONS)
		  {  
		     @sessNo = split(' ', $session );
		     $removed = `iimonitor $dbms << ! ;
remove $sessNo[0]
quit  
!`;   
		     # did it work?
		     $removed =~  s/IIMONITOR> //g ;
		     print $removed;
	    
		  }
	       }  
	    }
   	 }
      
      }
      else
      {
   	 print "No sessions\n";
      }    
   }  
} # end sub printServers

